package lezione2.clientserver.server;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
/**
 * Semplice server che manda indietro in output tutto ciò che riceve in input.
 * @see ServerSocket
 * @author Raffaele Giordanelli
 *
 */
public class EchoServer {
	/**
	 * Porta assegnata al servizio, al di fuori del range 1-1024
	 */
	public static final int PORT = 1050;

	public static void main(String[] args) throws IOException {
		ServerSocket serverSocket = new ServerSocket(PORT);
		System.out.println("EchoServer: started ");
		System.out.println("Server Socket: " + serverSocket);
		Socket clientSocket = null;
		BufferedReader in = null;
		PrintWriter out = null;
		try {
			// bloccante finchè non avviene una connessione
			clientSocket = serverSocket.accept();
			System.out.println("Connection accepted: " + clientSocket);
			// creazione stream di input da clientSocket
			InputStreamReader isr = new InputStreamReader(clientSocket.getInputStream());
			in = new BufferedReader(isr);
			// creazione stream di output su clientSocket
			OutputStreamWriter osw = new OutputStreamWriter(clientSocket.getOutputStream());
			BufferedWriter bw = new BufferedWriter(osw);
			out = new PrintWriter(bw, true);
			// ciclo di ricezione dal client e invio di risposta
			while (true) {
				String str = in.readLine();
				if (str.equals("END"))
					break;
				System.out.println("Echoing: " + str);
				out.println(str);
				out.flush(); //riga aggiunta per il flushing del buffer
			}
		} catch (IOException e) {
			System.err.println("Accept failed");
			System.exit(1);
		}
		// chiusura di stream e socket
		System.out.println("EchoServer: closing...");
		out.close();
		in.close();
		clientSocket.close();
		serverSocket.close();
	}

}
