package lezione2.multithread.client;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;

/**
 * Thread usato da EchoMultiClient.
 * 
 * @see Socket
 * @see Thread
 * @see EchoMultiClient
 * 
 * @author Raffaele Giordanelli
 * 
 */
public class ClientThread extends Thread {
	/**
	 * Socket usato dal Thread.
	 */
	private Socket socket;
	/**
	 * BufferedReader creato dal Socket.
	 */
	private BufferedReader in;
	/**
	 * PrintWriter creato dal Socket.
	 */
	private PrintWriter out;
	/**
	 * Contatore dei Threads creati.
	 */
	private static int counter = 0;
	/**
	 * ID del Thread corrente.
	 */
	private int id = counter++;
	/**
	 * Contatore dei Threads ATTUALMENTE in esecuzione.
	 */
	private static int threadcount = 0;

	/**
	 * @return il numero dei Threads ATTUALMENTE in esecuzione.
	 */
	public static int threadCount() {
		return threadcount;
	}

	/**
	 * Costruttore del Thread.
	 * 
	 * @param addr
	 *            l'indirizzo del server
	 */
	public ClientThread(InetAddress addr) {
		threadcount++;
		try {
			socket = new Socket(addr, 1050);
			System.out.println("EchoClient n° " + id + ": started");
			System.out.println("Client Socket: " + socket);
		} catch (IOException e) {
		}
		// creazione dei flussi dati
		try {
			InputStreamReader isr = new InputStreamReader(socket.getInputStream());
			in = new BufferedReader(isr);
			OutputStreamWriter osw = new OutputStreamWriter(socket.getOutputStream());
			out = new PrintWriter(new BufferedWriter(osw), true);
			// avvio del thread
			start();
		} catch (IOException e1) {
			// in seguito ad ogni fallimento la socket deve essere chiusa,
			// altrimenti
			// verrà chiusa dal metodo run() del thread
			try {
				socket.close();
			} catch (IOException e2) {
			}
		}
	}

	/**
	 * Attività del Thread. Invia 10 messaggi al server e attende le relative
	 * risposte.
	 * 
	 * @see Thread
	 */
	@Override
	public void run() {
		try {
			for (int i = 0; i < 10; i++) {
				out.println("client " + id + " msg " + i);
				System.out.println("Msg sent: client " + id + " msg" + i);
				String str = in.readLine();
				System.out.println("Echo: " + str);
			}
			out.println("END");
			out.flush();	//riga aggiunta per il flushing del buffer
		} catch (IOException e) {
		}
		System.out.println("Client " + id + " closing...");
		try {
			socket.close();
		} catch (IOException e) {
		}
		// il thread si chiude, quindi decremento il contatore dei threads in
		// esecuzione
		threadcount--;
	}

}
