package lezione2.multithread.server;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;

public class ServerThread extends Thread {
	/**
	 * Contatore dei thread server avviati.
	 */
	private static int counter = 0;
	/**
	 * ID del thread corrente.
	 */
	private int id = ++counter;
	/**
	 * Socket usato dal Thread.
	 */
	private Socket socket;
	/**
	 * BufferedReader creato dal Socket.
	 */
	private BufferedReader in;
	/**
	 * PrintWriter creato dal Socket.
	 */
	private PrintWriter out;

	/**
	 * Costruttore del Thread. Crea i flussi dati ed avvia l'attività del
	 * Thread.
	 * 
	 * @param s
	 *            la connessione con il client
	 * @throws IOException
	 */
	public ServerThread(Socket s) throws IOException {
		socket = s;
		in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
		OutputStreamWriter osw = new OutputStreamWriter(socket.getOutputStream());
		out = new PrintWriter(new BufferedWriter(osw), true);
		start();
		System.out.println("ServerThread " + id + ": started");
		System.out.println("Socket: " + s);
	}

	/**
	 * Attività del Thread. Riceve messaggi dal client e li reinvia indietro
	 * finché non riceve la stringa "END".
	 * 
	 * @see Thread
	 */
	@Override
	public void run() {
		try {
			while (true) {
				String str = in.readLine();
				if (str.equals("END"))
					break;
				System.out.println("ServerThread " + id + ": echoing -> " + str);
				out.println(str);
				out.flush(); //riga aggiunta per il flushing del buffer
			}
		} catch (IOException e) {
		}
		System.out.println("ServerThread " + id + ": closing...");
		try {
			socket.close();
		} catch (IOException e) {
		}
	}

}
