package lezione2.udpline.client;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.Socket;

/**
 * Invia 200 richieste al server, e stampa a schermo le 200 risposte.
 * L'indirizzo del server può essere opzionalmente specificato come parametro
 * del programma, altrimenti viene utilizzato il loopback IP address.
 * 
 * @see Socket
 * @see InetAddress
 * @author Raffaele Giordanelli
 * 
 */
public class LineClient {
	public static void main(String[] args) throws IOException {
		// creazione della socket datagram con un timeout di 30s
		DatagramSocket socket = new DatagramSocket();
		socket.setSoTimeout(30000);
		System.out.println("LineClient: started");
		System.out.println("Socket: " + socket);
		// creazione e invio del pacchetto di richiesta
		byte[] buf = new byte[256];
		InetAddress addr;
		if (args.length == 0)
			addr = InetAddress.getByName(null);
		else
			addr = InetAddress.getByName(args[0]);
		for (int i = 0; i < 200; i++) {
			DatagramPacket packet = new DatagramPacket(buf, buf.length, addr, 4444);
			socket.send(packet);
			System.out.println("Request sent to " + addr);
			// pulizia del buffer
			for (int j = 0; j < 256; j++)
				buf[j] = ' ';
			// ricezione e stampa della risposta
			packet = new DatagramPacket(buf, buf.length);
			socket.receive(packet);
			// sospensiva solo per i millisecondi indicati nel timeout,
			// dopodichè solleva una SocketException
			String received = new String(packet.getData());
			System.out.println("Line got: " + received);
		}
		System.out.println("LineClient: closing...");
		socket.close();
	}
}
