package lezione2.udpline.server;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.util.Date;

/**
 * Semplice server UDP che, ad ogni richiesta, invia la riga successiva di un
 * file. Se il file non viene trovato, invia semplicemente l'ora corrente.
 * 
 * @see DatagramSocket
 * @see DatagramPacket
 * @author Raffaele Giordanelli
 * 
 */
public class LineServer {
	/**
	 * Porta assegnata al servizio, al di fuori del range 1-1024
	 */
	public static final int PORT = 4444;
	/**
	 * Nome del file da leggere.
	 */
	public static final String FILE = "file.txt";
	/**
	 * DatagramSocket usata per ricevere i pacchetti UDP.
	 */
	public DatagramSocket socket;
	/**
	 * BufferedReader per leggere il contenuto del file.
	 */
	public static BufferedReader in = null;
	/**
	 * Indica se la lettura del file è terminata.
	 */
	public static boolean moreLines = true;

	/**
	 * Costruttore del LineServer, avvia il DatagramSocket per ricevere i
	 * pacchetti e il BufferedReader per leggere il file.
	 */
	public LineServer() {
		try {
			// creazione della socket datagram
			socket = new DatagramSocket(PORT);
			System.out.println("Socket: " + socket);
		} catch (SocketException e) {
			System.err.println("Unable to create the socket");
		}
		try {
			// associazione di uno stream di input al file da cui estrarre le
			// linee
			in = new BufferedReader(new FileReader(FILE));
			System.out.println("File " + FILE + " opened");
		} catch (FileNotFoundException e) {
			System.err.println("Could not open");
		}
	}

	public static void main(String[] args) throws IOException {
		System.out.println("LineServer: started");

		LineServer server = new LineServer();

		while (moreLines) {
			try {
				byte[] buf = new byte[256];
				// ricezione della richiesta
				DatagramPacket packet = new DatagramPacket(buf, buf.length);
				server.socket.receive(packet);
				// preparazione della linea da inviare
				String dString = null;
				if (in == null)// allora invio l'ora corrente
					dString = new Date().toString();
				else
					dString = getNextLine();
				if (dString != null) {
					buf = dString.getBytes();
					// "impacchettamento" e invio della risposta
					InetAddress address = packet.getAddress();
					int port = packet.getPort();
					packet = new DatagramPacket(buf, buf.length, address, port);
					server.socket.send(packet);
					System.out.println("Sending packet to " + address + ", " + port);
				}
			} catch (IOException e) {
				e.printStackTrace();
				moreLines = false;
			}
		}
		System.out.println("No more lines. Goodbye.");
		System.out.println("LineServer: closing...");
		server.socket.close();
	}

	/**
	 * @return la prossima linea del file specificato.
	 */
	public static String getNextLine() {
		String returnValue = null;
		try {
			if ((returnValue = in.readLine()) == null) {
				in.close();
				moreLines = false;
				returnValue = null;
			}
		} catch (IOException e) {
			returnValue = "IOException occurred.";
		}
		return returnValue;
	}

}
