package lezione3.multicastline.client;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

import lezione3.multicastline.server.MulticastLineServer;

/**
 * Esempio multicast. La classe crea un MulticastSocket, si iscrive al gruppo
 * multicast (indirizzo 230.0.0.1) e riceve 5 volte una linea di testo dal
 * server.
 * 
 * @see MulticastSocket
 * @author Raffaele Giordanelli
 * 
 */
public class MulticastLineClient {
	public static void main(String[] args) throws IOException {
		// creazione della socket multicast
		MulticastSocket socket = new MulticastSocket(MulticastLineServer.OUTPUTPORT);
		socket.setSoTimeout(20000);
		InetAddress address = InetAddress.getByName("230.0.0.1");
		System.out.println("MulticastClient: started");
		System.out.println("Socket: " + socket);
		// adesione al gruppo associato all'indirizzo multicast
		socket.joinGroup(address);
		System.out.println("Joining to " + address);
		DatagramPacket packet;

		// ricezione di alcune linee
		for (int i = 0; i < 5; i++) {
			byte[] buf = new byte[256];
			packet = new DatagramPacket(buf, buf.length);
			socket.receive(packet);
			String received = new String(packet.getData());
			System.out.println("Line got: " + received);
		}
		// uscita dal gruppo e chiusura della socket
		System.out.println("Leaving group...");
		socket.leaveGroup(address);
		System.out.println("MulticastClient: closing...");
		socket.close();
	}
}
