package lezione3.multicastline.server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.util.Date;

import lezione2.udpline.server.LineServer;

/**
 * Esempio multicast. La classe invia ogni secondo una linea di un file di testo
 * a tutti i client iscritti al gruppo multicast (indirizzo 230.0.0.1). Da
 * notare l'estensione della classe LineServer, poiché funzionalmente questo
 * server esegue lo stesso compito della sua superclasse, ciò che cambia è
 * l'indirizzo del destinatario dei pacchetti (che è un gruppo multicast).
 * 
 * @see LineServer
 * @author Raffaele Giordanelli
 * 
 */
public class MulticastLineServer extends LineServer {
	public static int OUTPUTPORT = 10000;

	public MulticastLineServer() {
		super();
	}

	public static void main(String[] args) throws IOException {
		long WAIT = 1000;
		int count = 0;
		System.out.println("MulticastServer: started");
		MulticastLineServer server = new MulticastLineServer();
		// creazione del gruppo associato all'indirizzo multicast
		InetAddress group = InetAddress.getByName("230.0.0.1");
		System.out.println("Creating group " + group);
		while (moreLines) {
			try {
				count++;
				byte[] buf = new byte[256];
				String dString = null;
				if (in == null)
					dString = new Date().toString();
				else
					dString = getNextLine();
				if (dString != null) {
					buf = dString.getBytes();
					DatagramPacket packet = new DatagramPacket(buf, buf.length, group, OUTPUTPORT);
					server.socket.send(packet);
					System.out.println("Sending line # " + count);
				}
				try {
					Thread.sleep((long) (Math.random() * WAIT));
				} catch (InterruptedException e) {
				}
			} catch (IOException e) {
				moreLines = false;
			}
		}
		System.out.println("No more lines. Goodbye.");
		System.out.println("MulticastServer: closing...");
		server.socket.close();
	}
}