package lezione5.crittografia;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.security.KeyPair;
import java.security.KeyPairGenerator;

import javax.crypto.Cipher;

/**
 * Questo esempio cifra e decifra la stringa inserita da tastiera, utilizzando
 * l'algoritmo a chiave pubblica Rivest Shamir Adleman (RSA)
 * 
 * @see KeyPairGenerator
 * @see Cipher
 * @author Raffaele Giordanelli
 * 
 */
public class ChiavePubblica {

	public static void main(String[] args) throws Exception {
		BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Insert a string");
		byte[] plainText = stdIn.readLine().getBytes("UTF8");

		// genera una coppia di chiavi RSA
		System.out.println("\nStart generating RSA key");
		KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
		keyGen.initialize(1024);
		KeyPair key = keyGen.generateKeyPair();
		System.out.println("Finish generating RSA key");

		// crea l'oggetto Cipher e stampa a schermo i dettagli della classe
		// provider dell'algoritmo
		Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
		System.out.println("\n" + cipher.getProvider().getInfo());

		// effettua la cifratura utilizzando la chiave PUBBLICA
		System.out.println("\nStart encryption");
		cipher.init(Cipher.ENCRYPT_MODE, key.getPublic());
		byte[] cipherText = cipher.doFinal(plainText);
		System.out.println("Finish encryption: ");
		System.out.println(new String(cipherText, "UTF8"));

		// effettua la decifratura utilizzando la chiave PRIVATA
		System.out.println("\nStart decryption");
		cipher.init(Cipher.DECRYPT_MODE, key.getPrivate());
		byte[] newPlainText = cipher.doFinal(cipherText);
		System.out.println("Finish decryption: ");
		System.out.println(new String(newPlainText, "UTF8"));
	}
}