package lezione5.crittografia;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;

import javax.crypto.Cipher;

/**
 * Questo esempio dimostra la tecnica della firma digitale, ottenuta generando
 * il digest del testo inserito, e cifrandolo con la chiave RSA privata, per
 * generare la firma. Per verificare la firma, si genera di nuovo il digest del
 * messaggio in chiaro, e lo si confronta con la decifratura della firma tramite
 * la chiave pubblica RSA. Se i due digest coincidono, la firma è verificata.
 * 
 * @author Raffaele Giordanelli
 * 
 */
public class FirmaDigitale {
	public static void main(String[] args) throws Exception {
		BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
		System.out.println("Insert a string");
		byte[] plainText = stdIn.readLine().getBytes("UTF8");

		// crea un oggetto Digest MD5 e calcola il digest della stringa inserita
		MessageDigest messageDigest = MessageDigest.getInstance("MD5");
		System.out.println("\n" + messageDigest.getProvider().getInfo());
		messageDigest.update(plainText);
		byte[] md = messageDigest.digest();
		System.out.println("\nDigest: ");
		System.out.println(new String(md, "UTF8"));

		// genera una coppia di chiavi RSA
		System.out.println("\nStart generating RSA key");
		KeyPairGenerator keyGen = KeyPairGenerator.getInstance("RSA");
		keyGen.initialize(1024);
		KeyPair key = keyGen.generateKeyPair();
		System.out.println("Finish generating RSA key");

		// crea un Cipher RSA
		Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
		System.out.println("\n" + cipher.getProvider().getInfo());

		// cifra il digest con la chiave PRIVATA RSA creando così la firma
		System.out.println("\nStart encryption");
		cipher.init(Cipher.ENCRYPT_MODE, key.getPrivate());
		byte[] cipherText = cipher.doFinal(md);
		System.out.println("Finish encryption: ");
		System.out.println(new String(cipherText, "UTF8"));

		// per verificare, decifra la firma utilizzando la chiave PUBBLICA RSA
		System.out.println("\nStart decryption");
		cipher.init(Cipher.DECRYPT_MODE, key.getPublic());
		byte[] newMD = cipher.doFinal(cipherText);
		System.out.println("Finish decryption: ");
		System.out.println(new String(newMD, "UTF8"));

		// poi rigenera il digest dal testo in chiaro, per simulare ciò che deve
		// fare un destinatario
		System.out.println("\nStart signature verification");
		messageDigest.reset();
		messageDigest.update(plainText);
		byte[] oldMD = messageDigest.digest();

		// infine verifica che i due digest coincidano
		int len = newMD.length;
		if (len > oldMD.length) {
			System.out.println("Signature failed, length error");
			System.exit(1);
		}
		for (int i = 0; i < len; ++i)
			if (oldMD[i] != newMD[i]) {
				System.out.println("Signature failed, element error");
				System.exit(1);
			}
		System.out.println("Signature verified");
	}
}