package lezione7.pki.decodifica;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.Cipher;

public class DecodificaMessaggio {
	public static void main(String args[]) throws Exception {
		codifica();
	}

	private static void codifica() throws Exception {

		// LEGGI CHIAVE PRIVATA CODIFICATA IN PKCS#8
		BufferedReader br_1 = new BufferedReader(new InputStreamReader(System.in));
		System.out.print("Inserire il nome del file contenete la chiave privata del destinatario: ");
		String privateKeyFile = br_1.readLine();
		byte[] privateKeyBytes = getBytes(privateKeyFile);
		// CONVERTI CHIAVE PRIVATA PKCS8 IN CHIAVE NORMALE
		// Inizializza convertitore da PKCS8 a chiave privata
		PKCS8EncodedKeySpec ks_1 = new PKCS8EncodedKeySpec(privateKeyBytes);
		// Inizializza un KeyFactory per ricreare la chiave usando RSA
		KeyFactory kf_1 = KeyFactory.getInstance("RSA");
		// Crea una chiave privata usando generatePrivate di KeyFactory in base
		// la chiave decodificata da ks_1
		PrivateKey privateKey = kf_1.generatePrivate(ks_1);

		// LEGGI CHIAVE PUBBLICA CODIFICATA IN X509
		BufferedReader br_2 = new BufferedReader(new InputStreamReader(System.in));
		System.out
				.print("\nInserire il nome del file contenuto nel \"PublicKeyStore\" contenete la chiave publica del mittente: ");
		String publicKeyFile = br_2.readLine();
		byte[] publicKeyBytes = getBytes(publicKeyFile);
		// CONVERTI CHIAVE PUBBLICA DA X509 A CHIAVE UTILIZZABILE
		// Inizializza convertitore da X.509 a chiave pubblica
		X509EncodedKeySpec ks_2 = new X509EncodedKeySpec(publicKeyBytes);
		// Inizializza un KeyFactory per ricreare la chiave usando RSA
		KeyFactory kf_2 = KeyFactory.getInstance("RSA");
		// Crea una chiave pubblica usando generatePublic di KeyFactory in base
		// la chiave decodificata da ks_2
		PublicKey publicKey = kf_2.generatePublic(ks_2);

		// LEGGI FILE SORGENTE DA DECODIFICARE
		BufferedReader br_3 = new BufferedReader(new InputStreamReader(System.in));
		System.out.print("\nInserire path file da decodificare, tra \" se contiene spazi: ");
		String sorgente = br_3.readLine();
		byte[] encryptFile = getBytes(sorgente);

		// DECODIFICA FILE SORGENTE
		System.out.print("\nInizio decodifica");
		// Inizializzo un cifrario che usa come algoritmo RSA, come modalità ECB
		// e come padding PKCS1
		Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
		System.out.print(".");

		// DECODIFICA DEL FILE CON LA CHIAVE PRIVATA DEL DESTINATARIO (PASSO 5 -
		// PRIMA DECODIFICA)

		// Lo inizializzo dicendo modalità di decodifica e chiave privata da
		// usare
		cipher.init(Cipher.DECRYPT_MODE, privateKey);
		System.out.print(".");

		// decodifico e metto il risultato in dencodeFilePrimaDecodifica
		byte[] dencodeFilePrimaDecodifica = new byte[encryptFile.length];
		int ixplain_1 = 0;
		int ixcode_1 = 0;
		while ((encryptFile.length - ixcode_1) > 128) {
			ixplain_1 += cipher.doFinal(encryptFile, ixcode_1, 128, dencodeFilePrimaDecodifica, ixplain_1);
			ixcode_1 += 128;
		}
		ixplain_1 += cipher.doFinal(encryptFile, ixcode_1, encryptFile.length - ixcode_1, dencodeFilePrimaDecodifica,
				ixplain_1);

		// ATTENZINE!!! La dimensione reale del testo decodificato è
		// "ixplain_1", quindi la dimensione di dencodeFilePrimaDecodifica va
		// ridotta a "ixplain_1" tutto quello che si trova oltre "ixplain_1" va
		// eliminato.
		byte[] tempDencodeFilePrimaDecodifica = new byte[ixplain_1];
		for (int i_1 = 0; i_1 < ixplain_1; i_1++) {
			tempDencodeFilePrimaDecodifica[i_1] = dencodeFilePrimaDecodifica[i_1];
		}
		dencodeFilePrimaDecodifica = tempDencodeFilePrimaDecodifica;

		// DECODIFICA DEL FILE CON LA CHIAVE PUBLICA DEL MITTENTE (PASSO 5 -
		// SECONDA DECODIFICA)

		// Lo inizializzo dicendo modalità di decodifica e chiave publica da
		// usare
		cipher.init(Cipher.DECRYPT_MODE, publicKey);
		System.out.print(".");

		// decodifico e metto il risultato in dencodeFileSecondaDecodifica
		byte[] dencodeFileSecondaDecodifica = new byte[dencodeFilePrimaDecodifica.length];
		int ixplain_2 = 0;
		int ixcode_2 = 0;
		while ((dencodeFilePrimaDecodifica.length - ixcode_2) > 128) {
			ixplain_2 += cipher.doFinal(dencodeFilePrimaDecodifica, ixcode_2, 128, dencodeFileSecondaDecodifica,
					ixplain_2);
			ixcode_2 += 128;
		}
		ixplain_2 += cipher.doFinal(dencodeFilePrimaDecodifica, ixcode_2, dencodeFileSecondaDecodifica.length
				- ixcode_2, dencodeFileSecondaDecodifica, ixplain_2);

		// ATTENZINE!!! La dimensione reale del testo decodificato è
		// "ixplain_2", quindi la dimensione di dencodeFileSecondaDecodifica va
		// ridotta a "ixplain_2" tutto quello che si trova oltre "ixplain_2" va
		// eliminato.
		byte[] tempDencodeFileSecondaDecodifica = new byte[ixplain_2];
		for (int i_2 = 0; i_2 < ixplain_2; i_2++) {
			tempDencodeFileSecondaDecodifica[i_2] = dencodeFileSecondaDecodifica[i_2];
		}
		dencodeFileSecondaDecodifica = tempDencodeFileSecondaDecodifica;

		System.out.println(". Codifica terminata!");

		// SALVA FILE CODIFICATO

		System.out.print("\nInserire path del file in cui salvare la codifica, tra \" se contiene spazi: ");
		BufferedReader br_4 = new BufferedReader(new InputStreamReader(System.in));
		String dest = br_4.readLine();
		FileOutputStream fos = new FileOutputStream(dest);
		fos.write(dencodeFileSecondaDecodifica);
		fos.close();
	}

	public static byte[] getBytes(String file) throws IOException {
		FileInputStream fis_1 = new FileInputStream(file);
		ByteArrayOutputStream baos_1 = new ByteArrayOutputStream();
		int i = 0;
		while ((i = fis_1.read()) != -1)
			baos_1.write(i);

		fis_1.close();
		byte[] privateKeyBytes = baos_1.toByteArray();
		baos_1.close();
		return privateKeyBytes;
	}
}
